@@ -3,21 +3,27 @@
 from __future__ import annotations
 
 import logging
+import traceback
 from typing import Any
+from urllib.parse import urlparse
 
 from mcp import Implementation, types
 from mcp.shared.exceptions import McpError
 from mcp_use.client import MCPClient as MCPUseClient
 from mcp_use.session import MCPSession as MCPUseSession
+from mcp_use.types.http import HttpOptions
 from pydantic import AnyUrl
 
+from hud.settings import settings
 from hud.types import MCPToolCall, MCPToolResult
+from hud.utils.hud_console import HUDConsole
 from hud.version import __version__ as hud_version
 
 from .base import BaseHUDClient
-from .utils.mcp_use_retry import patch_all_sessions
+from .utils.retry_transport import create_retry_httpx_client
 
 logger = logging.getLogger(__name__)
+hud_console = HUDConsole(logger=logger)
 
 
 class MCPUseHUDClient(BaseHUDClient):
@@ -27,7 +33,11 @@ class MCPUseHUDClient(BaseHUDClient):
         name="hud-mcp-use", title="hud MCP-use Client", version=hud_version
     )
 
-    def __init__(self, mcp_config: dict[str, dict[str, Any]] | None = None, **kwargs: Any) -> None:
+    def __init__(
+        self,
+        mcp_config: dict[str, dict[str, Any]] | None = None,
+        **kwargs: Any,
+    ) -> None:
         """
         Initialize MCP-use client.
 
@@ -48,6 +58,12 @@ class MCPUseHUDClient(BaseHUDClient):
             str, tuple[str, types.Tool, types.Tool]
         ] = {}  # server_name, original_tool, prefixed_tool
         self._client: Any | None = None  # Will be MCPUseClient when available
+        # Transport options for MCP-use (disable_sse_fallback, httpx_client_factory, etc.)
+        # Default to retry-enabled HTTPX client if factory not provided
+        self._http_options: HttpOptions = HttpOptions(
+            httpx_client_factory=create_retry_httpx_client,
+            disable_sse_fallback=True,
+        )
 
     async def _connect(self, mcp_config: dict[str, dict[str, Any]]) -> None:
         """Create all sessions for MCP-use client."""
@@ -55,18 +71,30 @@ class MCPUseHUDClient(BaseHUDClient):
             logger.warning("Client is already connected, cannot connect again")
             return
 
+        # If a server target matches HUD's MCP host and no auth is provided,
+        # inject the HUD API key as a Bearer token to avoid OAuth browser flow.
+        try:
+            hud_mcp_host = urlparse(settings.hud_mcp_url).netloc
+            if mcp_config and settings.api_key and hud_mcp_host:
+                for server_cfg in mcp_config.values():
+                    server_url = server_cfg.get("url")
+                    if not server_url:
+                        continue
+                    if urlparse(server_url).netloc == hud_mcp_host and not server_cfg.get("auth"):
+                        server_cfg["auth"] = settings.api_key
+        except Exception:
+            logger.warning("Failed to parse HUD MCP URL")
+
         config = {"mcpServers": mcp_config}
         if MCPUseClient is None:
             raise ImportError("MCPUseClient is not available")
-        self._client = MCPUseClient.from_dict(config)
+        self._client = MCPUseClient.from_dict(config, http_options=self._http_options)
         try:
-            assert self._client is not None  # noqa: S101
+            assert self._client is not None
             self._sessions = await self._client.create_all_sessions()
-            logger.info("Created %d MCP sessions", len(self._sessions))
-
-            # Patch all sessions with retry logic
-            patch_all_sessions(self._sessions)
-            logger.debug("Applied retry logic to all MCP sessions")
+            session_count = len(self._sessions)
+            session_text = "session" if session_count == 1 else "sessions"
+            hud_console.info(f"Created {session_count} MCP {session_text}")
 
             # Configure validation for all sessions based on client setting
             try:
@@ -86,21 +114,21 @@ class MCPUseHUDClient(BaseHUDClient):
             # Log session details in verbose mode
             if self.verbose and self._sessions:
                 for name, session in self._sessions.items():
-                    logger.debug("  - %s: %s", name, type(session).__name__)
+                    hud_console.debug(f"  - {name}: {type(session).__name__}")
 
         except McpError as e:
             # Protocol error - the server is reachable but rejecting our request
-            logger.error("MCP protocol error: %s", e)
-            logger.error("This typically means:")
-            logger.error("- Invalid or missing initialization parameters")
-            logger.error("- Incompatible protocol version")
-            logger.error("- Server-side configuration issues")
+            hud_console.warning(f"MCP protocol error: {e}")
+            hud_console.warning("This typically means:")
+            hud_console.warning("- Invalid or missing initialization parameters")
+            hud_console.warning("- Incompatible protocol version")
+            hud_console.warning("- Server-side configuration issues")
             raise
         except Exception as e:
             # Transport or other errors
-            logger.error("Failed to create sessions: %s", e)
+            hud_console.error(f"Failed to create sessions: {e}")
             if self.verbose:
-                logger.info("Check that the MCP server is running and accessible")
+                hud_console.info("Check that the MCP server is running and accessible")
             raise
 
         # Populate tool map during initialization
@@ -129,17 +157,14 @@ class MCPUseHUDClient(BaseHUDClient):
                     await session.initialize()
 
                 if session.connector.client_session is None:
-                    logger.warning("Client session not initialized for %s", server_name)
+                    hud_console.warning(f"Client session not initialized for {server_name}")
                     continue
 
                 # List tools (retry logic is handled at transport level)
                 tools_result = await session.connector.client_session.list_tools()
 
-                logger.info(
-                    "Discovered %d tools from '%s': %s",
-                    len(tools_result.tools),
-                    server_name,
-                    [tool.name for tool in tools_result.tools],
+                hud_console.info(
+                    f"Discovered {len(tools_result.tools)} tools from '{server_name}': {', '.join([tool.name for tool in tools_result.tools])}",  # noqa: E501
                 )
 
                 # Add to collections with optional prefix
@@ -167,16 +192,15 @@ class MCPUseHUDClient(BaseHUDClient):
                 if self.verbose:
                     for tool in tools_result.tools:
                         description = tool.description or ""
-                        logger.debug(
-                            "  Tool '%s': %s",
-                            tool.name,
-                            description[:100] + "..." if len(description) > 100 else description,
+                        hud_console.debug(
+                            f"  Tool '{tool.name}': {description[:100] + '...' if len(description) > 100 else description}",  # noqa: E501
                         )
 
             except Exception as e:
-                logger.error("Error discovering tools from '%s': %s", server_name, e)
+                hud_console.error(f"Error discovering tools from '{server_name}': {e}")
                 if self.verbose:
-                    logger.exception("Full error details:")
+                    hud_console.error("Full error details:")
+                    traceback.print_exc()
 
         return all_tools
 
@@ -196,12 +220,8 @@ class MCPUseHUDClient(BaseHUDClient):
         session = self._sessions[server_name]
 
         if self.verbose:
-            logger.debug(
-                "Calling tool '%s' (original: '%s') on server '%s' with arguments: %s",
-                tool_call.name,
-                original_tool.name,
-                server_name,
-                tool_call.arguments,
+            hud_console.debug(
+                f"Calling tool '{tool_call.name}' (original: '{original_tool.name}') on server '{server_name}' with arguments: {tool_call.arguments}"  # noqa: E501
             )
 
         if session.connector.client_session is None:
@@ -214,7 +234,7 @@ class MCPUseHUDClient(BaseHUDClient):
         )
 
         if self.verbose:
-            logger.debug("Tool '%s' result: %s", tool_call.name, result)
+            hud_console.debug(f"Tool '{tool_call.name}' result: {result}")
 
         # MCP-use already returns the correct type, but we need to ensure it's MCPToolResult
         return MCPToolResult(
@@ -223,8 +243,8 @@ class MCPUseHUDClient(BaseHUDClient):
             structuredContent=result.structuredContent,
         )
 
-    async def list_resources(self) -> list[types.Resource]:
-        """List all available resources."""
+    async def _list_resources_impl(self) -> list[types.Resource]:
+        """Implementation of resource listing for MCP-use client."""
         if self._client is None or not self._sessions:
             raise ValueError("Client is not connected, call initialize() first")
 
@@ -246,7 +266,7 @@ class MCPUseHUDClient(BaseHUDClient):
                 return resources.resources
             except Exception as e:
                 if self.verbose:
-                    logger.debug("Could not list resources from server '%s': %s", server_name, e)
+                    hud_console.debug(f"Could not list resources from server '{server_name}': {e}")
                 continue
         return []
 
@@ -276,8 +296,8 @@ class MCPUseHUDClient(BaseHUDClient):
                     raise AttributeError("read_resource not available")
 
                 if self.verbose:
-                    logger.debug(
-                        "Successfully read resource '%s' from server '%s'", uri, server_name
+                    hud_console.debug(
+                        f"Successfully read resource '{uri}' from server '{server_name}'"
                     )
 
                 return result
@@ -285,24 +305,21 @@ class MCPUseHUDClient(BaseHUDClient):
             except McpError as e:
                 # McpError is expected for unsupported resources
                 if "telemetry://" in str(uri):
-                    logger.debug(
-                        "Telemetry resource not supported by server '%s': %s", server_name, e
+                    hud_console.debug(
+                        f"Telemetry resource not supported by server '{server_name}': {e}"
                     )
                 elif self.verbose:
-                    logger.debug(
-                        "MCP resource error for '%s' from server '%s': %s", uri, server_name, e
+                    hud_console.debug(
+                        f"MCP resource error for '{uri}' from server '{server_name}': {e}"
                     )
                 continue
             except Exception as e:
                 # Other errors might be more serious
                 if "telemetry://" in str(uri):
-                    logger.debug("Failed to fetch telemetry from server '%s': %s", server_name, e)
+                    hud_console.debug(f"Failed to fetch telemetry from server '{server_name}': {e}")
                 else:
-                    logger.warning(
-                        "Unexpected error reading resource '%s' from server '%s': %s",
-                        uri,
-                        server_name,
-                        e,
+                    hud_console.warning(
+                        f"Unexpected error reading resource '{uri}' from server '{server_name}': {e}"  # noqa: E501
                     )
                 continue
 
@@ -311,14 +328,14 @@ class MCPUseHUDClient(BaseHUDClient):
     async def _disconnect(self) -> None:
         """Close all active sessions."""
         if self._client is None:
-            logger.warning("Client is not connected, cannot close")
+            hud_console.warning("Client is not connected, cannot close")
             return
 
         await self._client.close_all_sessions()
         self._sessions = {}
         self._tool_map = {}
         self._initialized = False
-        logger.debug("MCP-use client disconnected")
+        hud_console.debug("MCP-use client disconnected")
 
     # Legacy compatibility methods (limited; tests should not rely on these)
     def get_sessions(self) -> dict[str, Any]:
